# Plan
Here's a checklist of my immediate plans for the future. If you have an idea for a feature, feel free to make an issue.

## v1.1.0
* [x] Add *special areas*, static rectangles of arbitrary colour, position and size to the world. The server would choose where they are and they exist to hint to the user relevant locations used by server-side mods (like the existence of "portals" for teleportation of players).
	* [x] Render these special areas.
	* [x] Extend the protocol to support the server sending them.
		* [x] Change SERP Play request version for the development build.
	* [x] Add support for configuring these rectangles in the server config.
* [ ] ~Fix segfault when panicking on the client.~
	* [x] Update `egui_glium` in an attempt to fix the bug.
	* [ ] ~Use `EventLoop::run_app` instead of the deprecated `EventLoop::run` in an attempt to fix it.~
* [x] Get portals working through server-side configuration by using special areas and world PlayerPos/PlayerVel world events.
	* [x] Create the `PortalManager`, a struct on the server that handles teleportation, and hard-code a simple portal.
	* [x] Allow specifying portals through the server config as an arbitrary directed graph.
	* [x] Have the output position dependent on the input position.
	* [x] Prevent rapid teleportation with cycles of portals. Would prevent teleportation if the player is already colliding with a portal.
		* [x] Think about the case that a player spawns on a portal. Might need to prevent a sudden flash from the immediate teleportation. Also relevant for teleportation powerups.

		Won't yet fix for teleportation powerups.
	* [x] Allow overriding the default colours for each type of portal (sink/source/bidirectional), and for each portal.
	* [x] Render special areas below particle effects.
	* [x] Allow rotating the player's velocity when entering a portal.
	* [x] Put some work into preventing collisions with blocks when leaving a portal.
		* [x] Also consider the world border.
* [x] Bug fixes and polishes.
	* [x] Fix jitter on low delta time, with the player direction ~and colliding with blocks~.
		* [x] Direction problem.
		* [ ] ~Block problem.~
		Not bothering with block collision problem as the coefficient of restitution makes any jitter on high delta times (>150 from experience) unnoticeable, and fixing the bug is difficult to do without introducing new collision bugs.
	* [x] Fix bug with LAN discovery info send by the server not updating when the id changes.
	* [x] Create a `reload` command for the server REPL, which will be helpful when testing out a world.
* [x] Increase server-side customisability of effects.
	* [x] Update protocol to include the `PlayerEffect` world event to allow the server to choose when players get effects, and how long and how intense those effects are.
	* [x] Allow overriding the default effect multipliers and effect time in the server config.
		* [x] Also allow changing the default health added by health powerups.
		* [x] Allow customising the strength of the forcefield.
	* [x] Stack up effect times when collecting multiple powerups.
	* [x] Create effect zones, special areas where players get effects applied to them.
		* [x] Get a simple hard-coded effect zone working.
		* [x] Generalise it.
		* [x] Add to server config.
		* [x] Polish server config.
* [x] Ensure that collecting more than one ammo crates or powerups in an update results in both those being collected.
* [x] Avoid sending completely transparent special areas to the client.
* [x] Write some documentation.
	* [x] Copy over contents of `README.md`.
	* [x] Write even more.
	* [x] Update About game state to refer to this documentation.
	* [x] Use a custom theme.
* [x] Add a client-side limit for special areas.
* [x] Fix bug with the world border not rendering when one axis is infinite while another is finite.
* [x] Update `dist.sh` to build and include documentation.
* [x] Create some new server configs with these features.
	* [x] Portals.
	* [x] One central sink portal with many source portals around the map.
	* [x] Infinite, very fast reload, but weak bullets.
	* [x] Low reload but bullets instantly kill.
	* [x] Much slower and faster players than usual.
	* [x] Scarce bullets.
	* [x] High speed rail (fast speed effect zone).
	* [x] Regular map with many effect zones.
* [x] Prevent ammo crates and powerups from spawning in sink portals, as collecting them could be impossible.
* [x] Update SERP Play request version to 1.

## v1.1.1
These following changes would be implement through an `EffectController` struct.

* [ ] Have the way of handling multiple effects configurable (additive stacking, multiplicative stacking or adding the duration).
* [ ] Ensure that moving into an effect zone doesn't override any effects.

## v1.2.0
* [ ] Refactor module structure.
* [ ] Prevent players and powerups spawning too close to each other, to avoid an effect being applied for no apparent reason (if a powerup spawns directly on top of a player) which can be confusing (especially if it's a teleportation).
* [ ] Allow the player to choose their colour.
* [ ] Allow servers to specify some goal in the game and have it possible for players to win a game.
