// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use super::inkscape;
use super::image::Image;
use crate::utils;

use png::ColorType;
use static_assertions::const_assert;

pub fn generate() {
	const INPUT: &str = "data/block.svg";
	const INTERMEDIATE: &str = "/tmp/block.png";
	const OUTPUT: &str = "../data/textures/block.png";
	const BLOCK_IMAGE_TYPE: ColorType = ColorType::Rgb;

	const SIZE: u32 = 128;

	if !utils::newer(INPUT, OUTPUT) { return; }

	inkscape::gen_image(INPUT, INTERMEDIATE, SIZE, SIZE, BLOCK_IMAGE_TYPE);

	let mut image = Image::load(INTERMEDIATE, BLOCK_IMAGE_TYPE);

	assert!(image.width == SIZE && image.height == SIZE);
	const_assert!(SIZE % 4 == 0); // Might change this in the future

	for y in 0..SIZE / 2 {
		let mut src_i = (y * SIZE + y) as usize;
		for x in y..SIZE / 2 {
			let mut set = |x: u32, y: u32| {
				const CHANNEL_COUNT: usize = 3;
				assert!(BLOCK_IMAGE_TYPE == ColorType::Rgb); // If I change this in the future, the following function will break

				let dst_i = (x + y * image.width) as usize;
				for i in 0..CHANNEL_COUNT {
					image.data[dst_i * CHANNEL_COUNT + i] = image.data[src_i * CHANNEL_COUNT + i];
				}
			};

			//set(x, y);
			set(y, x);
			set(SIZE - 1 - x, y);
			set(SIZE - 1 - y, x);
			set(x, SIZE - 1 - y);
			set(y, SIZE - 1 - x);
			set(SIZE - 1 - x, SIZE - 1 - y);
			set(SIZE - 1 - y, SIZE - 1 - x);

			src_i += 1;
		}
	}

	image.save(OUTPUT, BLOCK_IMAGE_TYPE);
}
