// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use std::fs;
use std::path::Path;
use std::time::SystemTime;

use rand::RngCore;

pub trait Float: RngCore {
	fn next_f32(&mut self) -> f32 {
		self.next_u32() as f32 / u32::MAX as f32
	}
}

impl<T: RngCore> Float for T {}

fn modified_time(path: &str) -> Option<SystemTime> {
	fs::metadata(path).ok().map(|m| m.modified().expect("Use a better OS"))
}

// Checks if the input is newer than the output
// Panics if the input doesn't exist
pub fn newer(input: &str, output: &str) -> bool {
	modified_time(input).expect("Input file doesn't exist") > modified_time(output).unwrap_or(SystemTime::UNIX_EPOCH)
}

pub fn exists(path: &str) -> bool {
	Path::new(path).exists()
}
