// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use super::{ConnectionListener, IncomingConnection, DynIncomingConnection};

use crate::net::message_stream::{DynMs, singleplayer::LocalMessageStream};

pub struct SingleplayerConnectionListener(Option<LocalMessageStream>);

impl SingleplayerConnectionListener {
	pub fn new(stream: LocalMessageStream) -> SingleplayerConnectionListener {
		SingleplayerConnectionListener(Some(stream))
	}
}

impl ConnectionListener for SingleplayerConnectionListener {
	fn get(&mut self) -> Result<Option<Box<DynIncomingConnection>>, ()> {
		Ok(self.0.take().map(|x| -> Box<DynIncomingConnection> { Box::new(SingleplayerIncoming(x)) }))
	}
}

struct SingleplayerIncoming(LocalMessageStream);

impl IncomingConnection for SingleplayerIncoming {
	fn accept(self: Box<Self>) -> Result<Box<DynMs>, (String, String)> {
		Ok(Box::new(self.0))
	}

	fn reject(self: Box<Self>, _reason: String) {}
}
