// Copyright Marcus Del Favero 2024
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use strum_macros::{EnumCount, EnumIter};

use crate::game::config::action::Action;
use crate::utils::ToStr;

#[derive(Clone, Copy, EnumCount, EnumIter)]
pub(super) enum Category {
	Ui,
	Controls,
	Camera,
	Debug,
}

impl ToStr for Category {
	fn to_str(self) -> &'static str {
		match self {
			Category::Ui => "UI",
			Category::Controls => "Player Controls",
			Category::Camera => "Camera",
			Category::Debug => "Debug",
		}
	}
}

impl ToStr for Action {
	fn to_str(self) -> &'static str {
		match self {
			// UI
			Action::GuiLeave => "GUI Leave",
			Action::PlayingLeave => "Playing Leave",
			Action::ToggleFullscreen => "Fullscreen",

			// Controls
			Action::MoveLeft => "Left",
			Action::MoveRight => "Right",
			Action::MoveDown => "Down",
			Action::MoveUp => "Up",
			Action::Shoot => "Shoot",

			// Camera
			Action::ZoomIn => "Zoom In",
			Action::ZoomOut => "Zoom Out",
			Action::ZoomReset => "Reset Zoom",

			// Debug
			Action::PrintPos => "Print Position",
		}
	}
}

impl From<Action> for Category {
	fn from(action: Action) -> Category {
		match action {
			Action::GuiLeave | Action::PlayingLeave | Action::ToggleFullscreen => Category::Ui,
			Action::MoveLeft | Action::MoveRight | Action::MoveDown | Action::MoveUp | Action::Shoot => Category::Controls,
			Action::ZoomIn | Action::ZoomOut | Action::ZoomReset => Category::Camera,
			Action::PrintPos => Category::Debug,
		}
	}
}
